# -*- coding: utf-8 -*-

from spyne.model.primitive import Integer, String, Double, DateTime, Boolean, Time, Date
from spyne.model.complex import Iterable, Array, ComplexModel
import sys

if sys.version_info.major == 3:
    from spyne.const.xml import DEFAULT_NS
else:
    from spyne.const.xml_ns import DEFAULT_NS


class BaseAif(ComplexModel):
    __namespace__ = DEFAULT_NS


class DictConvertResult(BaseAif):
    result = Boolean
    info = String(encoding='utf8')


class AbnormalQuantityKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')


class AbnormalQuantity(AbnormalQuantityKey):
    minquant = Double(min_occurs=1, nillable=True)
    maxquant = Double(min_occurs=1, nillable=True)


# как минимум одно из полей обязательно для заполнения при удалении, а при добавлении обязательно поле number
class AccountpincodeKey(BaseAif):
    number = String(encoding='utf8')
    serverid = Integer


class Accountpincode(AccountpincodeKey):
    pincode = String(encoding='utf8')


class AccumulateSPrices(BaseAif):
    code = Integer(min_occurs=1, nillable=False)
    name = String(encoding='utf8')


class AdditionalCardKey(BaseAif):
    additionalnumber = String(min_occurs=1, nillable=False, encoding='utf8')
    idcardgroup = Integer(min_occurs=1, nillable=False)


class AdditionalCard(AdditionalCardKey):
    cardnumber = String(encoding='utf8')
    idclient = String(encoding='utf8')
    isactive = Integer(values=[0, 1])


class AgerestrictKey(BaseAif):
    alcoholpercent = Double(min_occurs=1, nillable=False)


class Agerestrict(AgerestrictKey):
    age = Integer(min_occurs=1, nillable=False)


class AlcoSetKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    alcosetbarcode = String(min_occurs=1, nillable=False, encoding='utf8')


class AlcoSet(AlcoSetKey):
    code = String(encoding='utf8')
    taracapacity = Double
    price = Double
    quant = Integer


class AspectKey(BaseAif):
    aspectcode = String(min_occurs=1, nillable=False, encoding='utf8')


class Aspect(AspectKey):
    aspectname = String(encoding='utf8')
    aspectorder = Integer
    aspectschemecode = Integer
    aspectvaluedesc = Integer


class AspectSchemeKey(BaseAif):
    aspectschemecode = Integer(min_occurs=1, nillable=False)


class AspectScheme(AspectSchemeKey):
    aspectschemename = String(encoding='utf8')


class AspectValueKey(BaseAif):
    aspectvaluecode = String(min_occurs=1, nillable=False, encoding='utf8')


class AspectValue(AspectValueKey):
    aspectvaluename = String(encoding='utf8')
    aspectcode = String(nillable=False, encoding='utf8')


class AspectValueSetKey(BaseAif):
    aspectvaluesetcode = Integer(min_occurs=1, nillable=False)


class AspectValueSet(AspectValueSetKey):
    aspectschemecode = Integer
    cquant = Double
    price = Double
    tmc_code = String(encoding='utf8')
    measure = Integer
    remain = Double
    name = String(encoding='utf8')
    remaindate = DateTime
    packingprice = Double
    productsectioncode = String(encoding='utf8')


class AspectValueSetValues(BaseAif):
    aspectcode = String(min_occurs=1, nillable=False, encoding='utf8')
    aspectvaluecode = String(min_occurs=1, nillable=False, encoding='utf8')
    aspectvaluesetcode = Integer(min_occurs=1, nillable=False)
    productsectioncode = String(encoding='utf8', nillable=False)
    tmc_code = String(encoding='utf8', nillable=False)


class BarcodeKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')


class Barcode(BarcodeKey):
    code = String(min_occurs=1, nillable=False, encoding='utf8')
    price = Double
    name = String(encoding='utf8')
    cquant = Double
    measure = Integer
    quantdefault = Double
    aspectvaluesetcode = Integer
    packingmeasure = Integer
    packingprice = Double
    minprice = Double
    minretailprice = Double
    customsdeclarationnumber = String(encoding='utf8')
    tmctype = Integer
    ntin = String(encoding='utf8')
    packagecode = String(encoding='utf8')


class ButtonKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class Button(ButtonKey):
    name = String(encoding='utf8')
    tag = String(encoding='utf8')
    rateobject = Integer
    ratetype = Integer
    ratevalue = Double
    maxratevalue = Double
    minratevalue = Double


class CardKey(BaseAif):
    idcard = String(min_occurs=1, nillable=False, encoding='utf8')


class CardOptions(BaseAif):
    disablerefundbonuses = Boolean


class Card(CardKey):
    idcardgroup = Integer
    number = String(min_occurs=1, nillable=False, encoding='utf8')
    blocked = Integer
    validitydatebeg = DateTime
    validitydateend = DateTime
    idclient = String(encoding='utf8')
    cardsum = Double
    lastknownbonusbalance = Double
    lastknownbonusbalancedate = DateTime
    multiplicator = Double
    multiplicatorbeg = DateTime
    multiplicatorend = DateTime
    multiplicatorcurrent = Double
    discountpercent = Double
    pincode = String(encoding='utf8')
    purchases = Integer
    cardstatus = Integer
    shopcode = String(encoding='utf8')
    options = CardOptions


class CardGroupKey(BaseAif):
    idcardgroup = Integer(min_occurs=1, nillable=False)


class CardGroup(CardGroupKey):
    name = String(encoding='utf8')
    pattern = String(encoding='utf8')
    regexrule = String(encoding='utf8')
    cardmode = Integer
    discounttype = Integer
    discountvalue = Double
    text = String(encoding='utf8')
    prefixbeg = String(encoding='utf8')
    prefixend = String(encoding='utf8')
    lenbeg = Integer
    lenend = Integer
    inputmask = Integer
    currencycode = Integer
    bonususecase = Integer
    showinfo = Integer
    notaddemptycard = Integer
    printinfo = Integer
    additionalregistration = Integer
    searchrule = Integer
    userawdata = Integer
    options = Integer


class CardGroupSearchPlaceKey(BaseAif):
    idcardgroup = Integer(min_occurs=1, nillable=False)


class CardGroupSearchPlace(CardGroupSearchPlaceKey):
    searchplace = String(encoding='utf8', min_occurs=1, nillable=False,
            values=['MAIN_CARD', 'ADDITIONAL_CARD', 'CARDSMOBILE'])
    name = String(encoding='utf8')
    extendedparams = String(encoding='utf8')


class CashUserKey(BaseAif):
    usercode = String(min_occurs=1, nillable=False, encoding='utf8')


class CashUser(CashUserKey):
    login = String(encoding='utf8')
    username = String(encoding='utf8')
    password = String(encoding='utf8')
    blocked = Integer
    userroleids = Array(Integer)


class CatalogKey(BaseAif):
    catalogid = Integer(min_occurs=1, nillable=False)


class Catalog(CatalogKey):
    catalogname = String(encoding='utf8')
    image = String(encoding='utf8')
    type = Integer


class CatalogAttributeKey(BaseAif):
    catalogid = Integer(min_occurs=1, nillable=False)


class CatalogAttribute(CatalogAttributeKey):
    attribute = Integer
    alcoholpercent = Double
    age = Integer
    egaistype = Integer


class ChangeAccountBalanceKey(BaseAif):
    sessionid = String(encoding='utf8', min_occurs=1, nillable=False)


class ChangeAccountBalance(ChangeAccountBalanceKey):
    accountnumber = String(encoding='utf8')
    cardnumber = String(encoding='utf8')
    changetype = String(encoding='utf8', min_occurs=1, nillable=False)
    value = Double(min_occurs=1, nillable=False)
    groupid = String(encoding='utf8')
    begin = DateTime
    end = DateTime
    weight = Integer
    serverid = Integer
    source = String(encoding='utf8')
    reason = String(encoding='utf8')


class ChangeCardAccountKey(BaseAif):
    cardnumber = String(encoding='utf8', min_occurs=1, nillable=False)


class ChangeCardAccount(ChangeCardAccountKey):
    cardstatus = String(encoding='utf8', min_occurs=1, nillable=False,
            values=['EARN_PAY', 'NO_OPERATION', 'EARN_ONLY', 'PAY_ONLY'])
    accountnumber = String(encoding='utf8')
    additionalstatus = String(encoding='utf8')
    serverid = Integer


class ClientKey(BaseAif):
    idclient = String(min_occurs=1, nillable=False)


class ClientOptions(BaseAif):
    sendchecktoemail = Integer
    emailconfirmed = Integer


class ClientCommunicationsOptions(BaseAif):
    call = Integer
    telegram = Integer
    email = Integer
    sms = Integer


class Client(ClientKey):
    name = String(encoding='utf8')
    text = String(encoding='utf8')
    sex = Integer
    birthday = DateTime
    specialdate1 = DateTime
    specialdate2 = DateTime
    specialdate3 = DateTime
    specialdate1name = String(encoding='utf8')
    specialdate2name = String(encoding='utf8')
    specialdate3name = String(encoding='utf8')
    zipcode = String
    address = String(encoding='utf8')
    email = String(encoding='utf8')
    webpage = String(encoding='utf8')
    phonenumber = String(encoding='utf8')
    inn = String
    document = String(encoding='utf8')
    okpo = String
    okpd = String
    occupation = String(encoding='utf8')
    childrencount = Integer
    extendedoptions = String(encoding='utf8')
    organizationcode = String(encoding='utf8')
    subscriptionadj = Integer
    options = ClientOptions
    lastname = String(encoding='utf8')
    patronymic = String(encoding='utf8')
    communications = ClientCommunicationsOptions


class OrganizationKey(BaseAif):
    organizationcode = String(encoding='utf8', min_occurs=1, nillable=False)


class Organization(OrganizationKey):
    name = String(min_occurs=1, nillable=False, encoding='utf8')
    registeredaddress = String(min_occurs=1, nillable=False, encoding='utf8')
    correspondenceaddress = String(encoding='utf8')
    ogrn = String(encoding='utf8')
    inn = String(min_occurs=1, nillable=False, encoding='utf8')
    kpp = String(min_occurs=1, nillable=False, encoding='utf8')
    okpo = String(encoding='utf8')
    operatingaccount = String(min_occurs=1, nillable=False, encoding='utf8')
    bank = String(min_occurs=1, nillable=False, encoding='utf8')
    correspondingaccount = String(min_occurs=1, nillable=False, encoding='utf8')
    bik = String(min_occurs=1, nillable=False, encoding='utf8')
    receiver = String(encoding='utf8')
    customer = String(encoding='utf8')
    payer = String(encoding='utf8')
    deliveryaddress = String(encoding='utf8')


class BackReasonKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class BackReason(BackReasonKey):
    doctype = Integer
    reason = String(encoding='utf8')


class CorrectionReasonKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class CorrectionReason(CorrectionReasonKey):
    reason = String(encoding='utf8')


class CmactionKey(BaseAif):
    cmactioncode = Integer(min_occurs=1, nillable=False)


class ActionParameter(BaseAif):
    parameterorder = Integer
    parametervalue = String(encoding='utf8')
    parametername = String(encoding='utf8')


class Cmaction(CmactionKey):
    cmactionname = String(encoding='utf8')
    commandcode = Integer
    notshowinservicemenu = Boolean
    actionparameters = Array(ActionParameter)


class CommandKey(BaseAif):
    commandcode = Integer(min_occurs=1, nillable=False)


class Command(CommandKey):
    commandname = String(encoding='utf8')
    description = String(encoding='utf8')


class DeptKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class Dept(DeptKey):
    name = String(encoding='utf8')
    idclient = String(encoding='utf8')
    documentprefix = String(encoding='utf8')
    merchantid = String(encoding='utf8')


class DiscCardAccumulate(BaseAif):
    disccode = Integer
    name = String(encoding='utf8')
    rtext = String(encoding='utf8')
    discperc = Double
    isbegsum = Integer
    begsum = Double
    isendsum = Integer
    endsum = Double
    isbegrcount = Integer
    begrcount = Double
    isendrcount = Integer
    endrcount = Double
    discscheme = Integer
    discschemename = String(encoding='utf8')


class ExciseMarkKey(BaseAif):
    excisemarkid = String(min_occurs=1, nillable=False, encoding='utf8')
    excisetype = String(encoding='utf8')


class ExciseMark(ExciseMarkKey):
    status = Integer


class ExciseMarkWhiteKey(BaseAif):
    excisemarkid = String(min_occurs=1, nillable=False, encoding='utf8')
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    excisetype = String(encoding='utf8')


class ExciseMarkWhite(ExciseMarkWhiteKey):
    shopcode = String(encoding='utf-8')
    inn = String(encoding='utf-8')
    serialnumber = String(encoding='utf-8')


class ExciseMarksWhite(BaseAif):
    excisemarkid = String(min_occurs=1, nillable=False, encoding='utf8')
    barcodes = Array(String(encoding='utf8'))
    excisetype = String(encoding='utf8')
    shopcode = String(encoding='utf-8')
    inn = String(encoding='utf-8')
    serialnumber = String(encoding='utf-8')


class HotKeyKey(BaseAif):
    hotkeycode = Integer(min_occurs=1, nillable=False)


class HotKey(HotKeyKey):
    hotkeyname = String(encoding='utf8')
    inventcodes = Array(String(encoding='utf8'))
    bybarcode = Boolean


################## Invent #####################
class InventKey(BaseAif):
    inventcode = String(min_occurs=1, nillable=False, encoding='utf8')


class RemainsOtions(BaseAif):
    traceremaindepartment = Integer
    modifyremaindepartment = Integer
    traceremainback = Integer
    manualremaintrace = Integer
    traceremainbycode = Integer


class QuantityOptions(BaseAif):
    enabledefaultquantity = Integer
    enablequantitymanual = Integer
    enablequantitybarcode = Integer
    enablequantityscales = Integer
    enablequantitylimit = Integer
    requirequantitymanual = Integer
    requirequantityscales = Integer
    requirequantitybarcode = Integer
    enabledocumentquantitylimit = Integer
    autogetquantityfromscales = Integer
    quantitylimit = Double
    documentquantlimit = Double


class InventItemOptions(BaseAif):
    enablebarcodemanual = Integer
    enablebarcodescanner = Integer
    disableinventsale = Integer
    disableinventback = Integer
    disableinventshow = Integer
    disablebackinsale = Integer
    visualverify = Integer
    ageverify = Integer
    requiresalerestrict = Integer
    enabledepartmentmanual = Integer
    requiredepartmentmanual = Integer
    egaisverify = Integer
    prepackaged = Integer
    nopdfegaisverify = Integer
    alcoset = Integer
    freesale = Integer
    rfidverify = Integer
    weightcontrolbypass = Integer
    lowweight = Integer
    tobacco = Integer
    shoes = Integer
    fuzzyweight = Integer
    ignoremarking = Integer
    markdownverify = Integer


class PriceOptions(BaseAif):
    enablepricemanual = Integer
    requirepricemanual = Integer
    requireselectprice = Integer
    requiredeferredprice = Integer
    enableexcisemarkprice = Integer


class Options(BaseAif):
    # remainsoptions = RemainsOtions
    quantityoptions = QuantityOptions
    inventitemoptions = InventItemOptions
    priceoptions = PriceOptions


class InventOptions(InventKey):
    options = Options


class AdditionalPriceKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    pricecode = Integer(min_occurs=1, nillable=False)
    effectivedate = DateTime(nillable=False)


class AdditionalPrice(AdditionalPriceKey):
    price = Double(min_occurs=1, nillable=False)
    name = String(encoding='utf8')
    documentid = String
    packingprice = Double


class SellRestrictPeriod(BaseAif):
    dateend = Date
    datestart = Date
    dayend = Integer
    daystart = Integer
    timeend = Time
    timestart = Time


class TmcKey(BaseAif):
    inventcode = String(min_occurs=1, nillable=False, encoding='utf8')


class Tmc(TmcKey):
    name = String(encoding='utf8')
    price = Double
    deptcode = Integer
    cquant = Double
    measurecode = Integer
    inventgroup = String(encoding='utf8')
    minprice = Double
    taxgroupcode = Integer
    extendedoptions = String(encoding='utf8')
    articul = String(encoding='utf8')
    remain = Double
    discautoscheme = Integer
    taracapacity = Double
    lvatcode = Array(Integer)
    defaultquantity = Double
    taramode = Integer
    aspectschemecode = Integer
    aspectvaluesetcode = Integer
    aspectusecase = Integer
    aspectselectionrule = Integer
    remaindate = DateTime
    age = Integer
    alcoholpercent = Double
    inn = String(encoding='utf8')
    kpp = String(encoding='utf8')
    options = Options
    alctypecode = Integer
    manufacturercountrycode = Integer
    paymentobject = Integer
    opmode = Integer
    loyaltymode = Integer
    minretailprice = Double
    ntin = String(encoding='utf8')
    packagecode = String(encoding='utf8')
    ownertype = Integer
    taraflag = Integer


class Invent(Tmc):
    barcode = String(encoding='utf8')
    barcodes = Array(Barcode)
    additionalprices = Array(AdditionalPrice)
    sellrestrictperiods = Array(SellRestrictPeriod)


class InventCatalog(CatalogKey):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')
    tmcprice = Double
    image = String(encoding='utf8')


class InventGroupKey(BaseAif):
    groupcode = String(min_occurs=1, nillable=False, encoding='utf8')


class InventGroup(InventGroupKey):
    parentgroupcode = String(encoding='utf8')
    groupname = String(encoding='utf8')


class KbKeyKey(BaseAif):
    keycode = Integer(min_occurs=1, nillable=False)
    context = String(encoding='utf8')


class KbKey(KbKeyKey):
    cmactioncode = Integer
    keyvalue = String(encoding='utf8')


class LDiscAccumulateKey(BaseAif):
    discscheme = Integer


class LDiscount(BaseAif):
    disccode = Integer(min_occurs=1, nillable=False)
    name = String(encoding='utf8')
    rtext = String(encoding='utf8')
    discperc = Double
    isbegsum = Integer
    begsum = Double
    isendsum = Integer
    endsum = Double
    isbegrcount = Integer
    begrcount = Integer
    isendrcount = Integer
    endrcount = Integer


class LDiscAccumulate(LDiscAccumulateKey):
    discschemename = String(encoding='utf8')
    discount = Array(LDiscount)


class LDiscCardKey(BaseAif):
    cardnumber = String(min_occurs=1, nillable=False, encoding='utf8')


class LDiscCard(LDiscCardKey):
    name = String(encoding='utf8')
    rtext = String(encoding='utf8')
    denycard = Integer
    acdiscscheme = Integer
    cardmode = Integer
    cardsum = Double
    discasheme = Integer
    acdiscasheme = Integer
    notaccumulate = Integer
    positionmode = Integer
    ispaycard = Integer
    allpositions = Integer
    msgsheme = Integer
    rcount = Double
    bonussum = Double
    bonuscode = Integer
    groupcode = Integer
    birthday = Date


class LDiscFixedKey(BaseAif):
    disccode = Integer(min_occurs=1, nillable=False)


class LDiscFixed(LDiscFixedKey):
    name = String(encoding='utf8')
    disctype = Integer
    discmode = Integer
    discsize = Double
    rtext = String(encoding='utf8')
    cardpref = String(encoding='utf8')
    receiptmode = Integer
    positionmode = Integer
    cardonly = Integer
    cardsumbeg = Double
    cardsumend = Double


class DiscountsAutoKeyLDisc(BaseAif):
    # key for command LDiscScheme
    disccode = Integer(min_occurs=1, nillable=False)


class DiscountsAutoKey(DiscountsAutoKeyLDisc):
    # keys for command DiscountsAuto
    schemecode = Integer(min_occurs=1, nillable=False)


class DiscountsAutoBase(DiscountsAutoKeyLDisc):
    name = String(encoding='utf8')
    disctype = Integer
    discmode = Integer
    discsize = Double
    rtext = String(encoding='utf8')
    datebeg = Date
    dateend = Date
    timebeg = Time
    timeend = Time
    daysbeg = Integer
    daysend = Integer
    quantbeg = Double
    quantend = Double
    sumbeg = Double
    sumend = Double
    pricecode = Integer
    rsummode = Integer
    cardgroupcode = Integer
    birthdaymode = Integer
    catalogid = Integer
    catalogquantbeg = Double
    catalogquantend = Double
    catalogsumbeg = Double
    catalogsumend = Double


class DiscountsAuto(DiscountsAutoBase):
    schemecode = Integer(min_occurs=1, nillable=False)


class LDiscSchemeKey(BaseAif):
    schemecode = Integer(min_occurs=1, nillable=False)


class LDiscScheme(LDiscSchemeKey):
    name = String(encoding='utf8')
    iscardscheme = Integer
    schemeoncheck = Integer
    discountsauto = Array(DiscountsAutoBase)


class LVatKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class LVat(LVatKey):
    name = String(encoding='utf8')
    rate = Double
    mode = Integer


class MCashUserKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')


class Roleuser(BaseAif):
    rolecode = Integer(min_occurs=1, nillable=False)
    rule = Integer


class MCashUser(MCashUserKey):
    login = String(encoding='utf8')
    name = String(encoding='utf8')
    password = String(encoding='utf8')
    locked = Integer
    roleusers = Array(Roleuser)
    keyposition = Integer
    rank = String(encoding='utf8')
    inn = String(encoding='utf8')


class MeasureKey(BaseAif):
    idmeasure = Integer(min_occurs=1, nillable=False)


class Measure(MeasureKey):
    name = String(encoding='utf8')
    isinteger = Boolean
    frunit = Integer


class MenuKey(BaseAif):
    menucode = Integer(min_occurs=1, nillable=False)


class MenuItem(BaseAif):
    menuitemorder = Integer
    cmactioncode = Integer


class Menu(MenuKey):
    menuname = String(encoding='utf8')
    menuitems = Array(MenuItem)


class PriceKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    documentid = String
    doctype = Integer


class PriceKeyByDate(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    effectivedate = DateTime
    effectivedateend = DateTime(nillable=True)


class OverduePriceKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')


class Price(PriceKey):
    price = Double(min_occurs=1, nillable=False)
    minprice = Double
    effectivedate = DateTime
    pricetype = Integer
    effectivedateend = DateTime(nillable=True)
    minretailprice = Double
    packingprice = Double


class RemainKey(BaseAif):
    documentid = String(min_occurs=1, nillable=False, encoding='utf8')
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    changetype = Integer(min_occurs=1, nillable=False)


class OutdatedRemainKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    effectivedate = DateTime(nillable=True)


class Remain(RemainKey):
    count = Integer(min_occurs=1, nillable=False)
    effectivedate = DateTime


class RoleKey(BaseAif):
    rolecode = Integer(min_occurs=1, nillable=False)


class RoleCommand(BaseAif):
    commandcode = Integer(min_occurs=1, nillable=False)
    documenttype = Integer(min_occurs=1, nillable=False)
    rule = Boolean(min_occurs=1, nillable=False)


class RoleRelation(BaseAif):
    secondaryrole = Integer(min_occurs=1, nillable=False)
    rule = Boolean(min_occurs=1, nillable=False)


class Role(RoleKey):
    rolename = String(encoding='utf8')
    rolecommands = Array(RoleCommand)
    rolerelations = Array(RoleRelation)


class ShopKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')


class Shop(ShopKey):
    extendedoptions = String(encoding='utf8')
    title = String(encoding='utf8')


class TaxGroupKey(BaseAif):
    idtaxgroup = Integer


class Tax(BaseAif):
    name = String(encoding='utf8')
    rate = Integer
    changebase = Boolean


class TaxGroup(TaxGroupKey):
    taxes = Array(Tax)


class TimerestrictKey(BaseAif):
    timerestrictid = Integer(min_occurs=1, nillable=False)


class Timerestrict(TimerestrictKey):
    alcoholpercent = Double
    datebeg = Date
    dateend = Date
    daysbeg = Integer
    daysend = Integer
    timebeg = Time
    timeend = Time


class CatalogRestrict(BaseAif):
    restrictid = Integer(min_occurs=1, nillable=False)
    catalogid = Integer
    deptcode = Integer


class UnitKey(BaseAif):
    unitcode = Integer(min_occurs=1, nillable=False)


class Unit(UnitKey):
    name = String(encoding='utf8')
    fractional = Boolean
    frunit = Integer


class CurrencyKey(BaseAif):
    currencycode = Integer(min_occurs=1, nillable=False)


class CurrencyStatus(BaseAif):
    basic = Boolean


class CurrencyMode(BaseAif):
    denypartialsum = Boolean
    denyextrasum = Boolean
    denymakechange = Boolean
    denypaymentforsale = Boolean
    denypaymentforback = Boolean
    printcopyrequired = Boolean
    denypaymentanydeptforbidded = Boolean
    paybyqrcodeonpinpad = Boolean
    manualamountrequired = Boolean
    paybycustomerqrcode = Boolean
    paybybiometry = Boolean
    showmoneyiteminfo = Boolean


class Currency(CurrencyKey):
    shortname = String(min_occurs=1, nillable=False, encoding='utf8')
    fullname = String(encoding='utf8')
    mode = CurrencyMode
    status = CurrencyStatus
    type = Integer(ge = 0, le = 9)
    decpoint = Integer
    round = String(encoding='utf8', values=['roundToUp', 'roundMathematical', 'roundMathematicalWithLoss',
            'roundToDown', 'roundToDownWithLoss'])
    drawer = Boolean
    hidden = Boolean
    operation = String(encoding='utf8', values=['none', 'certificate', 'processing'])
    payprocmodule = String(encoding='utf8')
    payprocdir = String(encoding='utf8')
    deptsforbiddance = Array(Integer)


class SoftCheckPosition(BaseAif):
    barcode = String(encoding='utf8')
    quantity = Double
    name = String(encoding='utf8')
    code = String(encoding='utf8')
    articul = String(encoding='utf8')
    measure = Integer
    cquant = Double
    quantitydefault = Double
    quantitylimit = Double
    group = String(encoding='utf8')
    documentdepartament = Integer
    deptnumber = Integer
    deptnumberall = Integer
    price = Double
    priceindexall = Integer
    pricetype = Integer
    pricevalcode = Integer
    minimumprice = Double
    priceselectmode = Integer
    sum = Double
    backmode = Integer
    isnotcalculate = Integer
    withoutcollection = Integer
    useaspects = Integer
    packingmode = Integer
    taraweight = Double
    vatcode1 = Integer
    vatrate1 = Double
    vatsum1 = Double
    vatcode2 = Integer
    vatrate2 = Double
    vatsum2 = Double
    vatcode3 = Integer
    vatrate3 = Double
    vatsum3 = Double
    vatcode4 = Integer
    vatrate4 = Double
    vatsum4 = Double
    vatcode5 = Integer
    vatrate5 = Double
    vatsum5 = Double
    tags = String(encoding='utf8')


class SoftCheckKey(BaseAif):
    softcheckid = String(encoding='utf8')


class SoftCheck(SoftCheckKey):
    options = Integer
    creationdate = DateTime
    positions = Array(SoftCheckPosition)


############# extended objects#################


class HotKeyInvent(InventKey):
    hotkeycode = Integer(min_occurs=1, nillable=False)


class KitInventKey(BaseAif):
    idkitgoods = Integer(min_occurs=1, nillable=False)


class KitKey(BaseAif):
    idkit = Integer(min_occurs=1, nillable=False)


class KitBase(KitKey):
    name = String(min_occurs=1, nillable=False, encoding='utf8')
    kittype = Integer
    maxtimes = Integer


class Kit(KitBase):
    iddiscountaction = Integer(min_occurs=1, nillable=False)


class KitDiscKey(BaseAif):
    idkitdisc = Integer(min_occurs=1, nillable=False)


class KitDiscBase(BaseAif):
    type = String(encoding='utf8', values=['percent', 'summ'])
    value = Double


class KitInventBase(BaseAif):
    inventcode = String(min_occurs=1, nillable=False, encoding='utf8')
    quantity = Double


class KitInventBaseDisc(KitInventBase):
    discountrate = KitDiscBase


class KitInvent(KitInventBase):
    idkit = Integer(min_occurs=1, nillable=False)


class KitDisc(KitDiscBase):
    idkitdisc = Integer(min_occurs=1, nillable=False)
    idkitgoods = Integer(min_occurs=1, nillable=False)


class KitBaseWithInvents(KitBase):
    invents = Array(KitInventBaseDisc)


class TimePeriodKey(BaseAif):
    idtimeperiod = Integer(min_occurs=1, nillable=False)


class TimePeriodBase(BaseAif):
    datestart = Date
    dateend = Date
    timestart = Time
    timeend = Time
    daystart = Integer
    dayend = Integer


class TimePeriod(TimePeriodBase):
    idtimeperiod = Integer(min_occurs=1, nillable=False)
    iddiscountaction = Integer(min_occurs=1, nillable=False)


class DiscountActionKey(BaseAif):
    iddiscountaction = Integer(min_occurs=1, nillable=False)


class DiscountAction(DiscountActionKey):
    name = String(encoding='utf8')
    timeperiods = Array(TimePeriodBase)
    kits = Array(KitBaseWithInvents)


class SaleRestrictKey(BaseAif):
    salerestrictcode = String(min_occurs=1, nillable=False, encoding='utf8')


class SaleRestrict(SaleRestrictKey):
    tmccode = String(min_occurs=1, encoding='utf8')
    dateend = Date
    datestart = Date
    dayend = Integer
    daystart = Integer
    timeend = Time
    timestart = Time


class ConsultantKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')


class Consultant(ConsultantKey):
    password = String(nillable=False, encoding='utf8')
    name = String(encoding='utf8')


class AlcoCode(BaseAif):
    alcocode = String(min_occurs=1, nillable=False, encoding='utf8')


class TmcAlcocode(AlcoCode):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')


class CertificateKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')


class Certificate(CertificateKey):
    rangefrom = Double(nillable=False)
    rangeto = Double(nillable=False)
    name = String(encoding='utf8')
    sum = Double
    inputmask = Integer(nillable=False)
    fullyspend = Boolean
    skiplastsymbol = Boolean
    tmccode = String
    minsum = Double
    maxsum = Double
    multiplesum = Double
    deptcode = Integer
    vatcode = Integer
    paymentobject = Integer
    regexpattern = String
    taxgroupcode = Integer
    discountcampaign = String(encoding='utf8')


class MacroParameters(BaseAif):
    macroparameterorder = Integer
    cmactioncode = Integer


class MacroKey(BaseAif):
    macrocode = Integer(min_occurs=1, nillable=False)


class Macro(MacroKey):
    macroname = String(encoding='utf8')
    macroparameters = Array(MacroParameters)


class EventKey(BaseAif):
    eventname = String(min_occurs=1, nillable=False, encoding='utf8')


class Event(EventKey):
    eventdescription = String(encoding='utf8')
    macrocode = Integer


class TmcScaleKey(BaseAif):
    tmcscalecode = String(min_occurs=1, nillable=False, encoding='utf8')
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')


class TmcScale(TmcScaleKey):
    tmcscalegroupcode = String(encoding='utf8')
    plu = Integer(min_occurs=1, nillable=False)
    ingredients = String(encoding='utf8')
    manufacturer = String(encoding='utf8')
    prefix = Integer
    produceddate = Integer
    sellbydate = Integer
    sellbytime = Integer


class TmcScaleGroupKey(BaseAif):
    tmcscalegroupcode = String(min_occurs=1, nillable=False, encoding='utf8')


class TmcScaleGroup(TmcScaleGroupKey):
    name = String(encoding='utf8')
    description = String(encoding='utf8')


class TmcRequiredQuantityKey(BaseAif):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')


class TmcRequiredQuantity(TmcRequiredQuantityKey):
    pass


class TmcVisualVerifyKey(BaseAif):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')


class TmcVisualVerify(TmcVisualVerifyKey):
    pass


class VerificationKey(BaseAif):
    idverification = Integer(min_occurs=1, nillable=False)


class Verification(VerificationKey):
    type = String(nillable=False, encoding='utf8')
    name = String(nillable=False, encoding='utf8')
    attribute = Integer()


class CardGroupVerification(BaseAif):
    idcardgroup = Integer(nillable=False)
    idverification = Integer(nillable=False)
    inputsource = Integer(nillable=False)
    extraverification = Boolean(nillable=True)


class CashConfigKey(BaseAif):
    name = String(min_occurs=1, nillable=False, encoding='utf8')


class CashConfig(CashConfigKey):
    value = String(nillable=False, encoding='utf8')


class ActionPanelKey(BaseAif):
    actionpanelcode = Integer(min_occurs=1, nillable=False)


class ActionPanelItem(BaseAif):
    actionpanelitemcode = Integer(nillable=False)
    actioncode = Integer(nillable=False)
    row = Integer(nillable=False)
    column = Integer(nillable=False)
    rowspan = Integer
    columnspan = Integer
    color = String(encoding='utf8')
    name = String(encoding='utf8')


class ActionPanel(ActionPanelKey):
    context = Integer(nillable=False)
    page = Integer(nillable=False)
    rowcount = Integer(nillable=False)
    columncount = Integer(nillable=False)
    actionpanelitems = Array(ActionPanelItem)


class CountryKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class Country(CountryKey):
    name = String(nillable=False, encoding='utf8')


class MedicineKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)


class Medicine(MedicineKey):
    party = String(nillable=True, encoding='utf8')
    barcode = String(nillable=False, encoding='utf8')
    series = String(nillable=True, encoding='utf8')
    shelflife = Date(nillable=True)
    producer = String(nillable=True, encoding='utf8')
    price = Double
    inn = String(nillable=True, encoding='utf8')
    remainquant = Integer(min_occurs=1, nillable=True)
    remaindatetime = DateTime(nillable=True)
    countrycode = Integer(min_occurs=1, nillable=True)
    options = Integer(min_occurs=1, nillable=True)
    packingprice = Double
    cquant = Double
    packquant = Double
    supplydate = Date(nillable=True)
    rating = Integer(nillable=True)
    bonusonsale = Integer(nillable=True)
    row = String(nillable=True, encoding='utf8')
    shelf = String(nillable=True, encoding='utf8')
    dictcode = String(nillable=True, encoding='utf8')
    dictname = String(nillable=True, encoding='utf8')


class SupplierKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')


class Supplier(SupplierKey):
    name = String(nillable=False, encoding='utf8')
    phone = String(nillable=False, encoding='utf8')
    inn = String(nillable=False, encoding='utf8')


class TmcSupplier(BaseAif):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')
    suppliercode = String(min_occurs=1, nillable=False, encoding='utf8')
    agentpaymentobject = Integer(min_occurs=1, nillable=False)


class TmcSupplierForDelBySomeField(BaseAif):
    tmccode = String(encoding='utf8')
    suppliercode = String(encoding='utf8')


class MCashUserAccess(BaseAif):
    code = String(nillable=False, encoding='utf8')
    shopcode = String(nillable=False, encoding='utf8')
    cashcode = String(nillable=False, encoding='utf8')


class PicklistKey(BaseAif):
    code = String(nillable=False, encoding='utf8')


class Picklist(PicklistKey):
    name = String(nillable=False, encoding='utf8')
    image = String(nillable=False, encoding='utf8')
    parent = String(encoding='utf8')
    tmccode = String(encoding='utf8')
    itemorder = Integer


class RemainTmcKey(BaseAif):
    identifier = String(nillable=False, encoding='utf8')
    shopcode = String(nillable=False, encoding='utf8')
    barcode = String(nillable=False, encoding='utf8')
    

class RemainTmc(RemainTmcKey):
    cquant = Double(nillable=False)
    changetype = Integer(nillable=False)
    effectivedate = DateTime(nillable=False)


class RemainBarcodeKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')


class RemainBarcode(RemainBarcodeKey):
    quant = Double(min_occurs=1, nillable=False)
    datetime = DateTime

class RemainDraftBeerKey(BaseAif):
    markingcode = String(min_occurs=1, nillable=False, encoding='utf8')

class RemainDraftBeer(RemainDraftBeerKey):
    barcode = String(min_occurs=1, encoding='utf8', nillable=False)
    name = String(min_occurs=1, encoding='utf8', nillable=False)
    volume = Double(min_occurs=1, nillable=False)
    connectdate = DateTime(min_occurs=1, nillable=False)
    expirationdate = DateTime(nillable=True)

class PersonalOfferKey(BaseAif):
    idclient = String(nillable=False, encoding='utf8')
    name = String(nillable=False, encoding='utf8')


class PersonalOffer(PersonalOfferKey):
    content = String(encoding='utf8', nillable=False)
    startdate = DateTime
    enddate = DateTime


class AgentKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')


class Agent(AgentKey):
    name = String(nillable=False, encoding='utf8')
    phone = String(nillable=False, encoding='utf8')
    inn = String(nillable=False, encoding='utf8')
    address = String(nillable=False, encoding='utf8')


class CategoryKey(BaseAif):
    categoryid = Integer(min_occurs=1, nillable=False)


class Category(CategoryKey):
    agentcode = String(nillable=False, encoding='utf8')
    categorycode = String(nillable=False, encoding='utf8')


class TmcKitKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')


class TmcKit(TmcKitKey):
    name = String(nillable=False, encoding='utf8')


class TmcKitItem(BaseAif):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')
    tmckitbarcode = String(min_occurs=1, nillable=False, encoding='utf8')
    quantity = Double

