# -*- coding: UTF-8 -*-

import sys
import os
from unload.node.node import Node
from unload.node.convertergetter import getConverterByName
from unload.node.nodeparams import DbReader, Cash
from unload.node.nodeparams import AifConverter, AifXmlConverter, SmagConverter, ShtrihConverter, OracleConverter, AtolAstorConverter, AtolConverter, CrystalConverter, NS2000Converter, PilotConverter, SapConverter, PilotOriginalConverter
from common import config, logger, macros, registry
from common.cashconvertersconfig import CashConvertersConfig

log = logger.getUnloadLogger("main")

shopCode = None
cashCode = None


def readStandartParams(section):

    salePath = config.getString(section, 'salePath', '/tmp/sales/')

    dirName = os.path.dirname(salePath)
    fileName = os.path.basename(salePath)
    dirName = macros.applyMacros(dirName, log, shopcode = shopCode, cashcode = cashCode)
    flagPath = macros.applyMacros(config.getString(section, 'flagPath', '/tmp/sales/sale.req'), log, shopcode = shopCode, cashcode = cashCode)
    rewriteDestFile = config.getBool(section, 'rewriteDestFile', False)
    return os.path.join(dirName, fileName), flagPath, rewriteDestFile

def getConverter(converterName):
    section = 'unload.converter.%s' % converterName

    if converterName != 'smag':
        onlineSalePath = macros.applyMacrosWithoutTime(config.getString(section, 'onlineSalePath', '/tmp/salesOnline/'), log, shopcode = shopCode, cashcode = cashCode)
    else:
        onlineSalePath = macros.applyMacros(config.getString(section, 'currentSalesPath', '/tmp/currentSales'), log, shopcode = shopCode, cashcode = cashCode)
    salePath, flagPath, rewriteDestFile = readStandartParams(section)

    cashConvertersConfig = CashConvertersConfig(config, section)

    return getConverterByName(converterName, cashConvertersConfig, salePath, flagPath, onlineSalePath, rewriteDestFile)

def getNodes():
    global cashCode
    global shopCode
    reg = registry.Registry()
    cashCode = str(reg.cashcode())
    shopCode = str(reg.shopcode())

    if not shopCode or shopCode == '':
        log.error("Cannot get shopCode from %s." % registry.Registry.path)
        sys.exit(1)

    configPath = os.path.normpath(sys.path[0] + "/../config/cash")
    configFiles = [configPath + "/exchangers.ini"]
    configPath += "/conf.d/"
    if os.path.isdir(configPath):
        for item in os.listdir(configPath):
            if os.path.isfile(configPath + item) and item.endswith(".ini"):
                configFiles.append(configPath + item)
    readFiles = config.read(configFiles)
    if not readFiles == configFiles:
        for fileName in configFiles:
            if fileName not in readFiles:
                log.error("Can't open configuration file \"%s\"" % fileName)
        sys.exit(1)

    convertersInConf = config.getString('UNLOAD', 'converter', 'aif').split(',')
    converters = []
    backupDir = config.getString('UNLOAD', 'backupDir', '/linuxcash/cash/data/backup/sales')
    maxCountFiles = config.getInt('UNLOAD', 'maxCountBackupFiles', 30)
    actualCountDays = config.getInt('UNLOAD', 'actualCountDaysForBackupFiles', 30)

    for converterName in convertersInConf:
        converterName = converterName.strip().lower()
        converter = getConverter(converterName)
        if converter:
            converters.append(converter)

    return [Node(id=shopCode,
            cashes=[Cash(cashCode, converters, backup_dir=backupDir, maxCountFiles=maxCountFiles, actualCountDays=actualCountDays)],
            reader=DbReader(host='localhost', port=3306, user='root', passwd='', db='documents'))]
