# -*- coding: utf-8 -*-

from aiohttp import web
from aiohttp.web_exceptions import HTTPUnauthorized
from aiohttp_spyne import AIOSpyne


class AIOSpyneWithAuth(AIOSpyne):
    
    def __init__(self, spyne_app, chunked = False, cache_wsdl = True, threads = None, authString = None):
        self.authString = authString
        AIOSpyne.__init__(self, spyne_app, chunked, cache_wsdl, threads)
    
    async def check_auth(self, request):
        if self.authString is not None and request.headers.get('AUTHORIZATION') != 'Basic %s' % self.authString:
            raise HTTPUnauthorized()
    
    async def post(self, request: web.Request) -> web.StreamResponse:
        await self.check_auth(request)
        return await AIOSpyne.post(self, request)
    
    async def get(self, request: web.Request) -> web.StreamResponse:
        await self.check_auth(request)
        return await AIOSpyne.get(self, request)

def startService(application, maxContentLength, port, authString):
    import multiprocessing
    handler = AIOSpyneWithAuth(application, threads=multiprocessing.cpu_count(), authString=authString)

    app = web.Application(client_max_size=maxContentLength)
    app.router.add_get("/{tail:.*}", handler.get)
    app.router.add_post("/{tail:.*}", handler.post)
    web.run_app(app, port=port)
